@@ -1,5 +1,5 @@
 # OfflineIMAP initialization code
-# Copyright (C) 2002-2011 John Goerzen & contributors
+# Copyright (C) 2002-2015 John Goerzen & contributors
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -23,13 +23,17 @@ import signal
 import socket
 import logging
 from optparse import OptionParser
+
 import offlineimap
-from offlineimap import accounts, threadutil, syncmaster
+from offlineimap import accounts, threadutil, syncmaster, folder
 from offlineimap import globals
-from offlineimap.error import OfflineImapError
 from offlineimap.ui import UI_LIST, setglobalui, getglobalui
 from offlineimap.CustomConfig import CustomConfigParser
 from offlineimap.utils import stacktrace
+from offlineimap.repository import Repository
+
+import traceback
+import collections
 
 
 class OfflineImap:
@@ -40,14 +44,17 @@ class OfflineImap:
       oi = OfflineImap()
       oi.run()
     """
+
     def run(self):
         """Parse the commandline and invoke everything"""
         # next line also sets self.config and self.ui
         options, args = self.__parse_cmd_options()
         if options.diagnostics:
             self.__serverdiagnostics(options)
+        elif options.migrate_fmd5:
+            self.__migratefmd5(options)
         else:
-            self.__sync(options)
+            return self.__sync(options)
 
     def __parse_cmd_options(self):
         parser = OptionParser(version=offlineimap.__version__,
@@ -57,108 +64,68 @@ class OfflineImap:
         parser.add_option("--dry-run",
                   action="store_true", dest="dryrun",
                   default=False,
-                  help="Do not actually modify any store but check and print "
-              "what synchronization actions would be taken if a sync would be"
-              " performed. It will not precisely give the exact information w"
-              "hat will happen. If e.g. we need to create a folder, it merely"
-              " outputs 'Would create folder X', but not how many and which m"
-              "ails it would transfer.")
+                  help="dry run mode")
 
         parser.add_option("--info",
                   action="store_true", dest="diagnostics",
                   default=False,
-                  help="Output information on the configured email repositories"
-              ". Useful for debugging and bug reporting. Use in conjunction wit"
-              "h the -a option to limit the output to a single account. This mo"
-              "de will prevent any actual sync to occur and exits after it outp"
-              "ut the debug information.")
+                  help="output information on the configured email repositories")
 
         parser.add_option("-1",
                   action="store_true", dest="singlethreading",
                   default=False,
-                  help="Disable all multithreading operations and use "
-              "solely a single-thread sync. This effectively sets the "
-              "maxsyncaccounts and all maxconnections configuration file "
-              "variables to 1.")
+                  help="(the number one) disable all multithreading operations")
 
         parser.add_option("-P", dest="profiledir", metavar="DIR",
-                  help="Sets OfflineIMAP into profile mode. The program "
-              "will create DIR (it must not already exist). "
-              "As it runs, Python profiling information about each "
-              "thread is logged into profiledir. Please note: "
-              "This option is present for debugging and optimization "
-              "only, and should NOT be used unless you have a "
-              "specific reason to do so. It will significantly "
-              "decrease program performance, may reduce reliability, "
-              "and can generate huge amounts of data. This option "
-              "implies the -1 option.")
+                  help="sets OfflineIMAP into profile mode.")
 
-        parser.add_option("-a", dest="accounts", metavar="ACCOUNTS",
-                  help="Overrides the accounts section in the config file. "
-              "Lets you specify a particular account or set of "
-              "accounts to sync without having to edit the config "
-              "file. You might use this to exclude certain accounts, "
-              "or to sync some accounts that you normally prefer not to.")
+        parser.add_option("-a", dest="accounts",
+                  metavar="account1[,account2[,...]]",
+                  help="list of accounts to sync")
 
         parser.add_option("-c", dest="configfile", metavar="FILE",
                   default=None,
-                  help="Specifies a configuration file to use")
+                  help="specifies a configuration file to use")
 
-        parser.add_option("-d", dest="debugtype", metavar="type1,[type2...]",
-                  help="Enables debugging for OfflineIMAP. This is useful "
-              "if you are to track down a malfunction or figure out what is "
-              "going on under the hood. This option requires one or more "
-              "debugtypes, separated by commas. These define what exactly "
-              "will be debugged, and so far include two options: imap, thread, "
-              "maildir or ALL. The imap option will enable IMAP protocol "
-              "stream and parsing debugging. Note that the output may contain "
-              "passwords, so take care to remove that from the debugging "
-              "output before sending it to anyone else. The maildir option "
-              "will enable debugging for certain Maildir operations. "
-              "The use of any debug option (unless 'thread' is included), "
-              "implies the single-thread option -1.")
+        parser.add_option("-d", dest="debugtype",
+                  metavar="type1[,type2[,...]]",
+                  help="enables debugging for OfflineIMAP "
+                  " (types: imap, maildir, thread)")
 
         parser.add_option("-l", dest="logfile", metavar="FILE",
-                  help="Log to FILE")
+                  help="log to FILE")
 
-        parser.add_option("-f", dest="folders", metavar="folder1,[folder2...]",
-                  help="Only sync the specified folders. The folder names "
-              "are the *untranslated* foldernames of the remote repository. "
-              "This command-line option overrides any 'folderfilter' "
-              "and 'folderincludes' options in the configuration file.")
+        parser.add_option("-s",
+                  action="store_true", dest="syslog",
+                  default=False,
+                  help="log to syslog")
+
+        parser.add_option("-f", dest="folders",
+                  metavar="folder1[,folder2[,...]]",
+                  help="only sync the specified folders")
 
         parser.add_option("-k", dest="configoverride",
                   action="append",
                   metavar="[section:]option=value",
-                  help=
-              """Override configuration file option. If"section" is
-              omitted, it defaults to "general". Any underscores
-              in the section name are replaced with spaces:
-              for instance, to override option "autorefresh" in
-              the "[Account Personal]" section in the config file
-              one would use "-k Account_Personal:autorefresh=30".""")
+                  help="override configuration file option")
 
         parser.add_option("-o",
                   action="store_true", dest="runonce",
                   default=False,
-                  help="Run only once, ignoring any autorefresh setting "
-                       "in the configuration file.")
+                  help="run only once (ignore autorefresh)")
 
         parser.add_option("-q",
                   action="store_true", dest="quick",
                   default=False,
-                  help="Run only quick synchronizations. Ignore any "
-              "flag updates on IMAP servers (if a flag on the remote IMAP "
-              "changes, and we have the message locally, it will be left "
-              "untouched in a quick run.")
+                  help="run only quick synchronizations (don't update flags)")
 
         parser.add_option("-u", dest="interface",
-                  help="Specifies an alternative user interface to "
-              "use. This overrides the default specified in the "
-              "configuration file. The UI specified with -u will "
-              "be forced to be used, even if checks determine that it is "
-              "not usable. Possible interface choices are: %s " %
-              ", ".join(UI_LIST.keys()))
+                  help="specifies an alternative user interface"
+                  " (quiet, basic, syslog, ttyui, blinkenlights, machineui)")
+
+        parser.add_option("--migrate-fmd5-using-nametrans",
+                  action="store_true", dest="migrate_fmd5", default=False,
+                  help="migrate FMD5 hashes from versions prior to 6.3.5")
 
         (options, args) = parser.parse_args()
         globals.set_options (options)
@@ -181,7 +148,7 @@ class OfflineImap:
         config = CustomConfigParser()
         if not os.path.exists(configfilename):
             # TODO, initialize and make use of chosen ui for logging
-            logging.error(" *** Config file '%s' does not exist; aborting!" %
+            logging.error(" *** Config file '%s' does not exist; aborting!"%
                           configfilename)
             sys.exit(1)
         config.read(configfilename)
@@ -194,14 +161,14 @@ class OfflineImap:
                 options.singlethreading = True
             if os.path.exists(options.profiledir):
                 # TODO, make use of chosen ui for logging
-                logging.warn("Profile mode: Directory '%s' already exists!" %
+                logging.warn("Profile mode: Directory '%s' already exists!"%
                              options.profiledir)
             else:
                 os.mkdir(options.profiledir)
             threadutil.ExitNotifyThread.set_profiledir(options.profiledir)
             # TODO, make use of chosen ui for logging
             logging.warn("Profile mode: Potentially large data will be "
-                         "created in '%s'" % options.profiledir)
+                         "created in '%s'"% options.profiledir)
 
         #override a config value
         if options.configoverride:
@@ -215,7 +182,7 @@ class OfflineImap:
                 config.set(section, key, value)
 
         #which ui to use? cmd line option overrides config file
-        ui_type = config.getdefault('general','ui', 'ttyui')
+        ui_type = config.getdefault('general', 'ui', 'ttyui')
         if options.interface != None:
             ui_type = options.interface
         if '.' in ui_type:
@@ -223,20 +190,20 @@ class OfflineImap:
             ui_type = ui_type.split('.')[-1]
             # TODO, make use of chosen ui for logging
             logging.warning('Using old interface name, consider using one '
-                            'of %s' % ', '.join(UI_LIST.keys()))
+                            'of %s'% ', '.join(UI_LIST.keys()))
         if options.diagnostics: ui_type = 'basic' # enforce basic UI for --info
 
-        #dry-run? Set [general]dry-run=True
+        # dry-run? Set [general]dry-run=True
         if options.dryrun:
-            dryrun = config.set('general','dry-run', "True")
-        config.set_if_not_exists('general','dry-run','False')
+            dryrun = config.set('general', 'dry-run', 'True')
+        config.set_if_not_exists('general', 'dry-run', 'False')
 
         try:
             # create the ui class
             self.ui = UI_LIST[ui_type.lower()](config)
         except KeyError:
-            logging.error("UI '%s' does not exist, choose one of: %s" % \
-                              (ui_type,', '.join(UI_LIST.keys())))
+            logging.error("UI '%s' does not exist, choose one of: %s"% \
+                              (ui_type, ', '.join(UI_LIST.keys())))
             sys.exit(1)
         setglobalui(self.ui)
 
@@ -244,6 +211,10 @@ class OfflineImap:
         if options.logfile:
             self.ui.setlogfile(options.logfile)
 
+        #set up syslog
+        if options.syslog:
+            self.ui.setup_sysloghandler()
+
         #welcome blurb
         self.ui.init_banner()
 
@@ -258,14 +229,17 @@ class OfflineImap:
                 options.singlethreading = True
 
             debugtypes = options.debugtype.split(',') + ['']
-            for type in debugtypes:
-                type = type.strip()
-                self.ui.add_debug(type)
-                if type.lower() == 'imap':
+            for dtype in debugtypes:
+                dtype = dtype.strip()
+                self.ui.add_debug(dtype)
+                if dtype.lower() == u'imap':
                     imaplib.Debug = 5
 
         if options.runonce:
-            # FIXME: maybe need a better
+            # Must kill the possible default option
+            if config.has_option('DEFAULT', 'autorefresh'):
+                config.remove_option('DEFAULT', 'autorefresh')
+            # FIXME: spaghetti code alert!
             for section in accounts.getaccountlist(config):
                 config.remove_option('Account ' + section, "autorefresh")
 
@@ -276,7 +250,7 @@ class OfflineImap:
         #custom folder list specified?
         if options.folders:
             foldernames = options.folders.split(",")
-            folderfilter = "lambda f: f in %s" % foldernames
+            folderfilter = "lambda f: f in %s"% foldernames
             folderincludes = "[]"
             for accountname in accounts.getaccountlist(config):
                 account_section = 'Account ' + accountname
@@ -308,6 +282,42 @@ class OfflineImap:
         self.config = config
         return (options, args)
 
+    def __dumpstacks(self, context=1, sighandler_deep=2):
+        """ Signal handler: dump a stack trace for each existing thread."""
+
+        currentThreadId = threading.currentThread().ident
+
+        def unique_count(l):
+            d = collections.defaultdict(lambda: 0)
+            for v in l:
+                d[tuple(v)] += 1
+            return list((k, v) for k, v in d.iteritems())
+
+        stack_displays = []
+        for threadId, stack in sys._current_frames().items():
+            stack_display = []
+            for filename, lineno, name, line in traceback.extract_stack(stack):
+                stack_display.append('  File: "%s", line %d, in %s'
+                                     % (filename, lineno, name))
+                if line:
+                    stack_display.append("    %s" % (line.strip()))
+            if currentThreadId == threadId:
+                stack_display = stack_display[:- (sighandler_deep * 2)]
+                stack_display.append('  => Stopped to handle current signal. ')
+            stack_displays.append(stack_display)
+        stacks = unique_count(stack_displays)
+        self.ui.debug('thread', "** Thread List:\n")
+        for stack, times in stacks:
+            if times == 1:
+                msg = "%s Thread is at:\n%s\n"
+            else:
+                msg = "%s Threads are at:\n%s\n"
+            self.ui.debug('thread', msg % (times, '\n'.join(stack[- (context * 2):])))
+
+        self.ui.debug('thread', "Dumped a total of %d Threads." %
+                      len(sys._current_frames().keys()))
+
+
     def __sync(self, options):
         """Invoke the correct single/multithread syncing
 
@@ -321,6 +331,8 @@ class OfflineImap:
             pass
 
         try:
+            # Honor CLI --account option, only.
+            # Accounts to sync are put into syncaccounts variable.
             activeaccounts = self.config.get("general", "accounts")
             if options.accounts:
                 activeaccounts = options.accounts
@@ -332,13 +344,13 @@ class OfflineImap:
             for account in activeaccounts:
                 if account not in allaccounts:
                     if len(allaccounts) == 0:
-                        errormsg = "The account '%s' does not exist because no"\
-                        " accounts are defined!" % account
+                        errormsg = "The account '%s' does not exist because no" \
+                            " accounts are defined!"% account
                     else:
-                        errormsg = "The account '%s' does not exist.  Valid ac"\
-                        "counts are: " % account
-                        errormsg += ", ".join(allaccounts.keys())
-                    self.ui.terminate(1, errormsg = errormsg)
+                        errormsg = "The account '%s' does not exist.  Valid ac" \
+                            "counts are: %s"% \
+                            (account, ", ".join(allaccounts.keys()))
+                    self.ui.terminate(1, errormsg=errormsg)
                 if account not in syncaccounts:
                     syncaccounts.append(account)
 
@@ -355,13 +367,22 @@ class OfflineImap:
                     getglobalui().warn("Terminating NOW (this may "\
                                        "take a few seconds)...")
                     accounts.Account.set_abort_event(self.config, 3)
+                    if 'thread' in self.ui.debuglist:
+                        self.__dumpstacks(5)
+
+                    # Abort after three Ctrl-C keystrokes
+                    self.num_sigterm += 1
+                    if self.num_sigterm >= 3:
+                        getglobalui().warn("Signaled thrice. Aborting!")
+                        sys.exit(1)
                 elif sig == signal.SIGQUIT:
-                    stacktrace.dump (sys.stderr)
+                    stacktrace.dump(sys.stderr)
                     os.abort()
 
-            signal.signal(signal.SIGHUP,sig_handler)
-            signal.signal(signal.SIGUSR1,sig_handler)
-            signal.signal(signal.SIGUSR2,sig_handler)
+            self.num_sigterm = 0
+            signal.signal(signal.SIGHUP, sig_handler)
+            signal.signal(signal.SIGUSR1, sig_handler)
+            signal.signal(signal.SIGUSR2, sig_handler)
             signal.signal(signal.SIGTERM, sig_handler)
             signal.signal(signal.SIGINT, sig_handler)
             signal.signal(signal.SIGQUIT, sig_handler)
@@ -380,12 +401,18 @@ class OfflineImap:
                                            'config': self.config})
                 t.start()
                 threadutil.exitnotifymonitorloop(threadutil.threadexited)
+
+            if not options.dryrun:
+                offlineimap.mbnames.write(True)
+
             self.ui.terminate()
+            return 0
         except (SystemExit):
             raise
         except Exception as e:
             self.ui.error(e)
             self.ui.terminate()
+            return 1
 
     def __sync_singlethreaded(self, accs):
         """Executed if we do not want a separate syncmaster thread
@@ -395,7 +422,7 @@ class OfflineImap:
         for accountname in accs:
             account = offlineimap.accounts.SyncableAccount(self.config,
                                                            accountname)
-            threading.currentThread().name = "Account sync %s" % accountname
+            threading.currentThread().name = "Account sync %s"% accountname
             account.syncrunner()
 
     def __serverdiagnostics(self, options):
@@ -407,3 +434,21 @@ class OfflineImap:
         for account in allaccounts:
             if account.name not in activeaccounts: continue
             account.serverdiagnostics()
+
+    def __migratefmd5(self, options):
+        activeaccounts = self.config.get("general", "accounts")
+        if options.accounts:
+            activeaccounts = options.accounts
+        activeaccounts = activeaccounts.replace(" ", "")
+        activeaccounts = activeaccounts.split(",")
+        allaccounts = accounts.AccountListGenerator(self.config)
+
+        for account in allaccounts:
+            if account.name not in activeaccounts:
+                continue
+            localrepo = Repository(account, 'local')
+            if localrepo.getfoldertype() != folder.Maildir.MaildirFolder:
+                continue
+            folders = localrepo.getfolders()
+            for f in folders:
+                f.migratefmd5(options.dryrun)
