@@ -1,5 +1,5 @@
 # Base repository support
-# Copyright (C) 2002-2012 John Goerzen & contributors
+# Copyright (C) 2002-2015 John Goerzen & contributors
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 
 import re
 import os.path
-import traceback
 from sys import exc_info
+
 from offlineimap import CustomConfig
 from offlineimap.ui import getglobalui
 from offlineimap.error import OfflineImapError
@@ -39,6 +39,7 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
         self.mapdir = os.path.join(self.uiddir, 'UIDMapping')
         if not os.path.exists(self.mapdir):
             os.mkdir(self.mapdir, 0o700)
+        # FIXME: self.uiddir variable name is lying about itself.
         self.uiddir = os.path.join(self.uiddir, 'FolderValidity')
         if not os.path.exists(self.uiddir):
             os.mkdir(self.uiddir, 0o700)
@@ -47,6 +48,7 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
         self.folderfilter = lambda foldername: 1
         self.folderincludes = []
         self.foldersort = None
+        self.newmail_hook = None
         if self.config.has_option(self.getsection(), 'nametrans'):
             self.nametrans = self.localeval.eval(
                 self.getconf('nametrans'), {'re': re})
@@ -103,9 +105,11 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
     def getmapdir(self):
         return self.mapdir
 
+    # Interface from CustomConfig.ConfigHelperMixin
     def getsection(self):
         return 'Repository ' + self.name
 
+    # Interface from CustomConfig.ConfigHelperMixin
     def getconfig(self):
         return self.config
 
@@ -116,7 +120,7 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
 
     def getlocaleval(self):
         return self.account.getlocaleval()
-    
+
     def getfolders(self):
         """Returns a list of ALL folders on this server."""
         return []
@@ -129,15 +133,25 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
     def getsep(self):
         raise NotImplementedError
 
+    def getkeywordmap(self):
+        raise NotImplementedError
+
+    def should_sync_folder(self, fname):
+        """Should this folder be synced?"""
+
+        return fname in self.folderincludes or self.folderfilter(fname)
+
     def get_create_folders(self):
         """Is folder creation enabled on this repository?
 
         It is disabled by either setting the whole repository
-        'readonly' or by using the 'createfolders' setting."""  
-        return self._readonly or self.getconfboolean('createfolders', True)
+        'readonly' or by using the 'createfolders' setting."""
+
+        return (not self._readonly) and \
+            self.getconfboolean('createfolders', True)
 
     def makefolder(self, foldername):
-        """Create a new folder"""
+        """Create a new folder."""
         raise NotImplementedError
 
     def deletefolder(self, foldername):
@@ -154,8 +168,9 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
         that forward and backward nametrans actually match up!
         Configuring nametrans on BOTH repositories therefore could lead
         to infinite folder creation cycles."""
+
         if not self.get_create_folders() and not dst_repo.get_create_folders():
-            # quick exit if no folder creation is enabled on either side.
+            # Quick exit if no folder creation is enabled on either side.
             return
 
         src_repo = self
@@ -163,7 +178,7 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
         dst_folders = dst_repo.getfolders()
         # Do we need to refresh the folder list afterwards?
         src_haschanged, dst_haschanged = False, False
-        # Create hashes with the names, but convert the source folders
+        # Create hashes with the names, but convert the source folders.
         # to the dest folder's sep.
         src_hash = {}
         for folder in src_folders:
@@ -176,17 +191,17 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
 
         # Find new folders on src_repo.
         for src_name_t, src_folder in src_hash.iteritems():
-            # Don't create on dst_repo, if it is readonly
+            # Don't create on dst_repo, if it is readonly.
             if not dst_repo.get_create_folders():
                 break
             if src_folder.sync_this and not src_name_t in dst_folders:
                 try:
                     dst_repo.makefolder(src_name_t)
-                    dst_haschanged = True # Need to refresh list
+                    dst_haschanged = True # Need to refresh list.
                 except OfflineImapError as e:
                     self.ui.error(e, exc_info()[2],
-                                  "Creating folder %s on repository %s" %\
-                                      (src_name_t, dst_repo))
+                         "Creating folder %s on repository %s"%
+                         (src_name_t, dst_repo))
                     raise
                 status_repo.makefolder(src_name_t.replace(dst_repo.getsep(),
                                                    status_repo.getsep()))
@@ -201,10 +216,10 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
                 # Does nametrans back&forth lead to identical names?
                 # 1) would src repo filter out the new folder name? In this
                 # case don't create it on it:
-                if not self.folderfilter(dst_name_t):
+                if not self.should_sync_folder(dst_name_t):
                     self.ui.debug('', "Not creating folder '%s' (repository '%s"
-                        "') as it would be filtered out on that repository." %
-                                  (dst_name_t, self))
+                        "') as it would be filtered out on that repository."%
+                        (dst_name_t, self))
                     continue
                 # get IMAPFolder and see if the reverse nametrans
                 # works fine TODO: getfolder() works only because we
@@ -254,3 +269,8 @@ class BaseRepository(CustomConfig.ConfigHelperMixin, object):
         for the threads to terminate."""
         pass
 
+    def getlocalroot(self):
+    	""" Local root folder for storing messages.
+    	Will not be set for remote repositories."""
+        return None
+
